(require 'cl)

(defcustom whitespace-files nil
  "List of file-specific behaviors for whitespace checking.

Each entry looks like (REGEXP CHECK t|nil ...).
REGEXP is a regexp that will be matched against the filename.
CHECK is one of 'leading, 'trailing, 'indent, 'spacetab or 'ateol. Checks not
specified will inherit from the default value."
  :type '(repeat
	  (group (regexp :tag "File Name Matching")
		 (repeat :inline t :tag "Checks"
			 (choice :inline t
				 (list :inline t :tag "Leading"
				       :format "%{%t%}: %v"
				       (const :tag "" :value leading)
				       boolean)
				 (list :inline t :tag "Trailing"
				       :format "%{%t%}: %v"
				       (const :tag "" :value trailing)
				       boolean)
				 (list :inline t :tag "Indent"
				       :format "%{%t%}: %v"
				       (const :tag "" :value indent)
				       boolean)
				 (list :inline t :tag "Space Tab"
				       :format "%{%t%}: %v"
				       (const :tag "" :value spacetab)
				       boolean)
				 (list :inline t :tag "At EOL"
				       :format "%{%t%}: %v"
				       (const :tag "" :value ateol)
				       boolean)))))
  :group 'whitespace)


(defun whitespace-check-whitespace-mode (&optional arg)
  "Test and set the whitespace-mode in \"qualifying\" buffers.
A buffer qualifies if its file name matches something in
\`whitespace-files', or if its mode matches something in
\`whitespace-modes'."
  (if (null whitespace-mode)
      ;; first, let's find a match on the filename
      (let* ((file-name (buffer-file-name))
	     (behavior (assoc* file-name whitespace-files
			       :test #'(lambda (f r) (string-match r f)))))
	(cond (behavior
	       (setq whitespace-mode t)
	       (pop behavior)
	       (let (symbol)
		 (while (setq symbol (pop behavior))
		   (set (intern (concat "whitespace-check-buffer-"
					(symbol-name symbol)))
			(pop behavior)))))
	      ;; otherwise, look at the major-mode
	      (t
	       (setq whitespace-mode
		     (if (or arg (member major-mode whitespace-modes))
			 t
		       nil)))))))
